<?php

namespace Alive\Dm;

/**
 * HTTP client for discount manager related calls.
 * It can be used only within wordpress because it's using wordpress functions to make HTTP calls.
 */
class Discount_Manager_Client
{

    private $dm_api_url;
    private $api_gateway_url;
    private $args;
    private $api_gateway_body_args;

    /**
     * Create new instance of the client.
     * @param string $url Url address of the discount manager. Never empty, never null.
     * @param string $username Username for the discount manager API. Never empty, never null.
     * @param string $password Password for discount manager API. Never empty, never null.
     * @param string $api_gateway_url Url address for API gateway. Never empty, never null.
     * @param string $api_access_key_id Access key id address for API gateway. Never empty, never null.
     * @param string $api_secret_access_key Secret access key for API gateway. Never empty, never null.
     */
    public function __construct($url, $username, $password, $api_gateway_url, $api_access_key_id, $api_secret_access_key)
    {
        $this->dm_api_url = rtrim($url, "/") . '/api';
        $this->args = array(
            'headers' => array(
                'Authorization' => $this->get_basic_authentication_header($username, $password),
            )
        );
        $this->api_gateway_url = rtrim($api_gateway_url, "/") . '/v1';
        $this->api_gateway_body_args = array(
            'access_key_id' => $api_access_key_id,
            'secret_access_key' => $api_secret_access_key,
        );
    }


    //~ Suggestions

    /**
     * Get suggestions for given term.
     * @param string $text Term to be searched for.
     * @return array Array representing the suggestions.
     */
    function get_suggestions($text)
    {
        return $this->get('/suggestions', null, array('text' => $text));
    }


    //~ Categories

    /**
     * Get all categories from DM.
     * @return array Arary of all categories.
     */
    function get_categories()
    {
        return $this->get('/categories');
    }

    /**
     * Get category by primary identifier.
     * @param int $category_id category identifier.
     * @return array categories.
     */
    function get_category($category_id)
    {
        return $this->get('/categories/{id}', array('id' => $category_id));
    }


    //~ providers

    /**
     * get provider by primary identifier.
     * @param unknown $provider_id Provider identifier.
     * @return array $provider Provider parameters.
     */
    function get_provider($provider_id)
    {
        $provider = $this->get('/providers/{id}', array('id' => $provider_id));
        if (is_object($provider) && $provider->visible) {
            $visibleBranches = array();
            foreach ($provider->branches as $branch) {
                if ($branch->visible) {
                    $visibleBranches[] = $branch;
                }
            }
            $provider->branches = $visibleBranches;
        }
        return is_object($provider) && $provider->visible ? $provider : null;
    }

    /**
     * Get comments of given provider.
     * @param unknown $provider_id Provider identifier.
     * @param unknown $offset Offset.
     * @param unknown $limit $limit.
     * @return array Comments.
     */
    function get_comments($provider_id, $offset, $limit)
    {
        return ($this->get('/providers/{id}/comments',
            array('id' => $provider_id), array('offset' => $offset, 'limit' => $limit)));
    }

    /**
     * Get consumer.
     * @param string $ext_id External identifier.
     * @return array Consumer.
     */
    function get_consumer($external_id)
    {
        $consumers = $this->get('/consumers', null, array('externalId' => $external_id));
        if (count($consumers->items) == 1) {
            return $consumers->items[0];
        }
        return null;
    }

    /**
     * Tady je škoda vůbec psát nějaký komentář.
     * @deprecated Návr a signatura téhle metody je špatně. Měla by přijmat objekt, a ne jej konstuovat.
     */
    function create_consumer($username, $ext_id, $token = null)
    {
        $consumer = array(
            'username' => $username,
            'externalId' => $ext_id,
            'token' => $token,
        );
        return $this->post('/consumers', null, $consumer);
    }

    /**
     * Replace consumer definition.
     * @param $consumer Consumer values.
     * @param integer $replace_by_external_id Consumer identifier.
     * @return multitype:NULL
     */
    function replace_consumer($consumer, $replace_by_external_id)
    {
        //FIXME: Tahle metoda by vůbec neměla sahat na objekt.
        $consumer = (array)$consumer;
        $consumer['replacedBy'] = array(
            'externalId' => $replace_by_external_id,
        );
        unset($consumer['createdOn']);
        return $this->put('/consumers/{id}', array('id' => $consumer['id']), $consumer);
    }

    /**
     * Create view of the provider.
     * @param integer $provider_id Provider identifier.
     */
    function add_provider_view($provider_id)
    {
        $providerView = array(
            'userAgent' => $_SERVER['HTTP_USER_AGENT'],
            'ip' => $_SERVER['REMOTE_ADDR'],
        );
        $this->post('/providers/{id}/views', array('id' => $provider_id), $providerView);
    }

    /**
     * Get ratings.
     * @param int $provider_id Provider identifier.
     * @param int $consumer_id Consumer identifier.
     * @return mixed|number
     */
    function get_ratings($provider_id, $consumer_id)
    {
        return $this->get('/providers/{id}/ratings', array('id' => $provider_id), array('consumerId' => $consumer_id));
    }

    /**
     * Tady je škoda vůbec psát nějaký komentář.
     * @deprecated Návr a signatura téhle metody je špatně. Měla by přijmat objekt, a ne jej konstuovat.
     */
    function create_comment($text, $ip, $user_agent, $provider_id, $consumer_id, $rating_id)
    {
        $comment = array(
            'text' => $text,
            'ip' => $ip,
            'userAgent' => $user_agent,
            'consumer' => array(
                'id' => $consumer_id,
            ),
        );
        if ($rating_id != null) {
            $comment['rating'] = array('id' => $rating_id);
        }
        return $this->post('/providers/{id}/comments', array('id' => $provider_id), $comment);
    }

    /**
     * Tady je škoda vůbec psát nějaký komentář.
     * @deprecated Návr a signatura téhle metody je špatně. Měla by přijmat objekt, a ne jej konstuovat.
     */
    function rate_provider($provider_id, $value, $consumer_id, $ip, $user_agent)
    {
        $rating = array(
            'value' => $value,
            'ip' => $ip,
            'userAgent' => $user_agent,
            'consumer' => array(
                'id' => $consumer_id,
            )
        );
        return $this->post('/providers/{id}/ratings', array('id' => $provider_id), $rating);
    }

    /**
     * \param $params mixed valid keys: name, status, sortKey, sortOrder, offset, limit
     */
    function get_providers($params)
    {
        $valid_keys = array('text', 'categoryIds', 'cardTypes', 'sortKey', 'sortOrder',
            'location', 'offset', 'limit', 'facets', 'seed', 'locale', 'newest');
        $queryParams = array('visible' => 'true');
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && $params[$key] !== null) {
                $queryParams[$key] = $params[$key];
            }
        }
        return $this->get('/providers', null, $queryParams);
    }


    //~ Branches

    /**
     * \param $params mixed valid keys: text, cardTypes, categories, boundingBox, availableOnline, facets, sortKey, sortOrder, location, offset, limit,
     */
    function get_branches($params)
    {
        $valid_keys = array('text', 'cardTypes', 'categoryIds', 'boundingBox', 'availableOnline', 'facets', 'sortKey', 'sortOrder', 'location', 'offset', 'limit');
        $queryParams = array('visible' => 'true');
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && $params[$key] !== null) {
                $queryParams[$key] = $params[$key];
            }
        }
        return $this->get('/branches', null, $queryParams);
    }

    //~ Get authentication token

    /**
     * Get authentication token of Verifications Service API
     */
    function get_authentication_token()
    {
        return $this->post('/authentication.authenticate', null, $this->api_gateway_body_args, $this->api_gateway_url, []);
    }


    //~ Generate discount url

    /**
     * After success card and discountId verification this function is called - it generates discount url
     * \param $params mixed valid keys: verification_id, discount_id
     */
    function generate_discount_url($params, $locale)
    {
        $valid_keys = array('verification_id', 'discount_id');
        $queryParams = array('visible' => 'true');
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && $params[$key] !== null) {
                $queryParams[$key] = $params[$key];
            }
        }

        $options = $this->args;
        $options['headers']['X-User-Lang'] = $locale;

        return $this->post('/discounts/generate-discount-url', null, $queryParams, $this->dm_api_url, $options);
    }

    //~ Card verification

    /**
     *
     * \param $params mixed valid keys: cardholder_name, card_number, provider_id
     */
    function get_card_verification($params)
    {
        $token = $this->get_authentication_token()->data->token;

        $options = [];
        $options['headers']['Authorization'] = 'Bearer ' . $token;

        $valid_keys = array('cardholder_name', 'card_number', 'provider_id');
        $queryParams = array();
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && $params[$key] !== null) {
                $queryParams[$key] = $params[$key];
            }
        }

        return $this->post('/verifications.verify', null, $queryParams, $this->api_gateway_url, $options);
    }

    //~ Request methods.

    /**
     * Make GET request to Discount manager.
     * @param string $path Path of the request. Never empty, never null.
     * @param array $pathParams Path parameters for the request. Can be empty.
     * @param array $query Query parameters of the request. Can be null.
     * @return mixed Response of the request.
     */
    private function get($path, $pathParams = array(), $query = array(), $baseUrl = null)
    {
        $response = $this->wp_request('GET', $path, $pathParams, $query, $baseUrl);
        return json_decode(wp_remote_retrieve_body($response));
    }

    /**
     * Make POST request.
     * @param string $path Path of the request.
     * @param array $pathParams Can be null.
     * @param array $body Never null.
     */
    private function post($path, $pathParams, $body, $baseUrl = null, $options = null)
    {
        $response = $this->wp_request('POST', $path, $pathParams, null, $body, $baseUrl, $options);
        $body = wp_remote_retrieve_body($response);
        if (!empty($body)) {
            return json_decode($body);
        }
        return null;
    }

    /**
     * Make PUT request.
     * @param string $path Path of the request.
     * @param array $pathParams Can be null.
     * @param array $body Never null.
     */
    private function put($path, $pathParams, $body, $baseUrl = null)
    {
        $response = $this->wp_request('PUT', $path, $pathParams, null, $body, $baseUrl);
        $body = wp_remote_retrieve_body($response);
        if (!empty($body)) {
            return json_decode($body);
        }
        return null;
    }

    /**
     * Make HTTP request with gicen params.
     * @param string $method Method to be used.
     * @param unknown $path Path of the request.
     * @param unknown $pathParams Path parameters.
     * @param unknown $query Query parameters.
     * @param string $body Body.
     */
    private function wp_request($method, $path, $pathParams, $query, $body = null, $baseUrl = null, $options = null)
    {
        if ($baseUrl === null) {
            $baseUrl = $this->dm_api_url;
        }

        $url = $this->constructUrl($baseUrl, $path, $pathParams, $query);
        // Construct request options.
        if ($options === null) {
            $options = $this->args;
        }

        $options['method'] = $method;
        if ($body !== null) {
            $options['body'] = json_encode($body); // We assume that all calls are JSON rot now.
        }
        $options['headers']['Content-Type'] = 'application/json';
        $response = wp_remote_request($url, $options);
        $responseCode = wp_remote_retrieve_response_code($response);
        if (substr($responseCode, 0, 1) == '2') {
            return $response;
        } else if ($responseCode == '400') {
            self::error("'$method' request to '$url' returned a 400 Bad Request response.", $response);
        } elseif ($response instanceof \WP_Error) {
            self::error(sprintf("Cannot make '%s' request to '%s'. Request failed with code '%s' and '%s'.", $method,
                $url, $response->get_error_code(), $response->get_error_message()), $response);
        } else {
            self::error("Cannot make '" . $method . "' request to '" . $url .
                "'. Response body: '" . wp_remote_retrieve_body($response) . "'.", $response);
        }
        return [];
    }

    /**
     * Construct URL based on the parameters.
     * @param strin $baseUrl Base URL.
     * @param string $path Path within the base URL.
     * @param array $pathParams Path parameters list.
     * @param array $query Query parameters.
     */
    private function constructUrl($baseUrl, $path, $pathParams, $query)
    {
        // Replace path parameters.
        if (!empty($pathParams)) {
            foreach ($pathParams as $key => $value) {
                $path = str_replace('{' . $key . '}', urldecode($value), $path);
            }
        }
        // Consturct URL.
        $url = $baseUrl . $path;
        // Add query.
        if (!empty($query)) {
            $url .= '?' . http_build_query($query);
        }
        return $url;
    }


    //~ Helper methods

    /**
     * Log error.
     * @param string $msg Message.
     * @param string $response response of the call.
     */
    private function error($msg, $response)
    {
        Log::error("$msg for: " . serialize($response));
    }

    /**
     * Get authentication header value for basic authentication.
     * @param string $username Username. Never null.
     * @param string $password Password. Never null.
     * @return Encoded basic authentication header. Never null.
     */
    private function get_basic_authentication_header($username, $password)
    {
        return 'Basic ' . base64_encode($username . ':' . $password);
    }

}
