<?php
namespace Alive\Dm;

use SimpleXMLElement;

class Ncdb_Client {

    const NCDB_XML_RPC_WS_ISSUER_PATH = '/ws/cardIssuer';
    const NCDB_XML_RPC_WS_REFERRAL = '/ws/referral';

    private $base_url;
    private $username;
    private $password;
    private $base_request_args;

    /**
     * Create new instance of the NCDB client.
     */
    public function __construct() {
        $this->username = Config::ncdb_username();
        $this->password = Config::ncdb_password();
        if (empty(Config::ncdb_url()) || empty(Config::ncdb_username()) || empty(Config::ncdb_password())) {
            $this->write_log('Error: Missing required parameters for NCDB client');
        }
        $headers = array(
                'Authorization' =>  $this->get_basic_authentication_header($this->username, $this->password),
        );
        $this->base_request_args = array(
                'headers' => $headers
        );
        $this->base_url = Config::ncdb_url();
    }

    /**
     * Get value of 'Authorization' header for basic authentication.
     * @param string $username Username. Never null.
     * @param string $password Password. Never null.
     */
    private function get_basic_authentication_header($username, $password) {
        return 'Basic ' . base64_encode($username . ':' . $password);
    }

    public function verify_card_number($card_number, $card_holder_name) {
        $params = array();
        $params[] = $card_number;
        $params[] = trim($card_holder_name);
        $params[] = null;
        $params[] = null;
        $response = $this->xml_rpc_request(self::NCDB_XML_RPC_WS_REFERRAL, 'Referral.useCard', $params);
        return $response;
    }

    /**
     * Make XML-RPC request.
     * @param string $service Location of the service. Never empty, never null.
     * @param string $method Name of the method to be called. Never empty, never null.
     * @param array $params Array of request parameters. Can be empty, never null.
     * @param string $account Account to be used within the request.
     * @return mixed Response.
     */
    private function xml_rpc_request($service, $method, $params = array()) {
        $request = xmlrpc_encode_request($method, $params, array('encoding' => 'utf-8', 'escaping' => 'markup'));
        $file = $this->make_xml_rpc_request($service, $request, $this->username, $this->password);
        $response = xmlrpc_decode($file, 'utf-8');
        return $response;
    }

    /**
     * Make request to the XML RPC
     *
     * @param string $service Name of the service to be called.
     * @param xml $requestBody Encoded body for xmlrpc request.
     * @param string $username Authentication username.
     * @param string $password Authentication password.
     * @return xml Encoded response of the  xmlrpc result.
     */
    private function make_xml_rpc_request($service, $requestBody, $username, $password) {
        $process = curl_init($this->base_url . $service);
        curl_setopt($process, CURLOPT_HTTPHEADER, array('Content-Type: application/xml', 'Content-Encoding: UTF-8'));
        curl_setopt($process, CURLOPT_HEADER, 0);
        curl_setopt($process, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($process, CURLOPT_TIMEOUT, 30);
        curl_setopt($process, CURLOPT_POST, 1);
        curl_setopt($process, CURLOPT_POSTFIELDS, $requestBody);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);
        $return = curl_exec($process);
        curl_close($process);
        return $return;
    }

    /**
     * @param $log
     */
    public function write_log ( $log )  {
        if ( true === WP_DEBUG ) {
            if ( is_array( $log ) || is_object( $log ) ) {
                error_log( print_r( $log, true ) );
            } else {
                error_log( $log );
            }
        }
    }
}
